@@ -26,7 +26,7 @@ pub struct ZestRand {
     // Seed of the CSPRNG (THIS INCLUDES THE COUNTER) -- higher bits get changed first
     seed: [u64; 62],
     // Current output of the CSPRNG
-    hash_output: Option<[u64; 64]>,
+    hash_output: [u64; 64],
     // Current byte of the CSPRNG (in the output)
     byte_offset: usize,
 }
@@ -36,25 +36,25 @@ impl ZestRand {
     pub fn seed(&mut self, seed: [u64; 62], byte_offset: usize) {
         if self.seed != seed {
             self.seed = seed;
-            self.hash_output = None;
+            self.hash_output = zest4096(self.generate_input());
         }
         self.byte_offset = byte_offset & 0x1FF;
     }
     // Counts to the next block, (lower index ptr = higher place value)
     // and sets the hash output to None
     #[inline(always)]
-    fn next_block(&mut self) {
+    pub fn next_block(&mut self) {
         let mut carry: u64 = 1;
         for elem in self.seed.iter_mut().rev() {
             *elem = (Wrapping(*elem) + Wrapping(carry)).0;
             carry = (*elem < carry) as u64;
         }
-        self.hash_output = None;
+        self.hash_output = zest4096(self.generate_input());
     }
-    // Hash the current block, return it for use.
+    // Get the current block
     #[inline(always)]
-    pub fn compute_block(&mut self) -> [u64; 64] {
-        zest4096(self.generate_input())
+    pub fn get_block(&mut self) -> [u64; 64] {
+        self.hash_output
     }
     // Generate the input to the zest hash based on seed, index, and magic bytes
     #[inline(always)]
@@ -74,11 +74,8 @@ impl ZestRand {
     // Fetch the next byte, this constructs everything.
     #[inline(always)]
     pub fn next_byte(&mut self) -> u8 {
-        if self.hash_output.is_none() {
-            self.hash_output = Some(self.compute_block());
-        }
         // Safety: known size and type before and after conversion
-        let slice_ref = &self.hash_output.unwrap();
+        let slice_ref = &self.hash_output;
         let cast_slice: &[u8; 512] = unsafe { transmute(slice_ref) };
         let byte_val = cast_slice[self.byte_offset & 0x1FF];
         self.byte_offset += 1;
@@ -103,7 +100,7 @@ impl ZestRand {
     /// Do not use with enums, as not all values could map to a valid enum.
     /// Think before you use!
     #[inline(always)]
-    pub unsafe fn fill_rand<A: Copy>(&mut self) -> A {
+    pub unsafe fn fill_rand<A: Copy + Sized>(&mut self) -> A {
         let mut data = vec![];
         data.resize_with(size_of::<A>(), || self.next_byte());
         let data_address = addr_of!(data[..]) as *const A;
@@ -139,7 +136,10 @@ impl Default for ZestRand {
     fn default() -> Self {
         // Safety: All fields are set to 0, and none
         // are pointers to any other object.
-        unsafe { zeroed() }
+        // The initial block is also generated.
+        let mut initial: Self = unsafe { zeroed() };
+        initial.hash_output = zest4096(initial.generate_input());
+        initial
     }
 }
 
@@ -169,8 +169,9 @@ impl Read for ZestRand {
     }
 }
 
-// Functions for shuffling slice types:
+// Functions for dealing with slice types:
 impl ZestRand {
+    // Shuffle a slice using non-biased uniform indexing.
     #[inline(always)]
     pub fn slice_shuffle<T: Sized + Copy>(&mut self, slice: &mut [T]) {
         for i in 0..slice.len() {
@@ -178,4 +179,23 @@ impl ZestRand {
             slice.swap(i, swap_index);
         }
     }
+    // Fill a vector with a number of random objects
+    /// # Safety
+    /// Do not use with objects containing pointers or references,
+    /// as they are written over with random data, and can cause a seg fault.
+    /// Do not use with enums, as not all values could map to a valid enum.
+    /// Think before you use!
+    #[inline(always)]
+    pub unsafe fn fill_vec_any<T: Sized + Copy>(&mut self, len: usize) -> Vec<T> {
+        let mut data: Vec<T> = vec![];
+        data.resize_with(len, || self.fill_rand());
+        data
+    }
+
+    // Fill a vector with random numbers.
+    // Wrapper around fill_vec_any.
+    #[inline(always)]
+    pub fn fill_vec_num<T: Sized + Copy + Integer>(&mut self, len: usize) -> Vec<T> {
+        unsafe { self.fill_vec_any(len) }
+    }
 }
